/**
 * <copyright>
 *
 * Service Data Objects
 * Version 3.0
 * Licensed Materials
 *
 * (c) Copyright BEA Systems, Inc., International Business Machines Corporation,
 * Oracle Corporation, Primeton Technologies Ltd., Rogue Wave Software, SAP AG.,
 * Software AG., Sun Microsystems, Sybase Inc., Xcalia, Zend Technologies,
 * 2005-2008. All rights reserved.
 *
 * </copyright>
 *
 */
package commonj.sdo.helper;

import java.util.Map;

/**
 * The HelperContextFactory is used to get instances of HelperContext. 
 * @since SDO 3.0
 */
public interface HelperContextFactory {

    /**
     * Create a new HelperContext in this implementation.  Once created the HelperContext
     * can be looked up as follows (Note if the identifier is null or "" it is not registered):
     * SDO.getHelperContext(identifier);
     * @param identifier - A unique identifier that can be used to access the HelperContext.
     * @param properties - Properties required to initialize the HelperContext.
     * @return a HelperContext object
     * @throws IllegalArgumentException If a different HelperContext is already 
     * registered with the specified identifier or invalid properties are specified.
     */
    public HelperContext createHelperContext(Map<String, Object> properties, String identifier) throws IllegalArgumentException;

    /**
     * Create a new HelperContext in this implementation with a unique generated
     * identifier.  The created HelperContext can be accessed using
     * SDO.getHelperContext(returnedHelperContext.getIdentifier());
     * @param properties - Properties required to initialize the HelperContext.
     * @return a HelperContext object
     * @throws IllegalArgumentException If invalid properties are specified.
     */
    public HelperContext createHelperContext(Map<String, Object> properties) throws IllegalArgumentException;

}