/**
 * <copyright>
 *
 * Service Data Objects
 * Version 3.0
 * Licensed Materials
 *
 * (c) Copyright BEA Systems, Inc., International Business Machines Corporation,
 * Oracle Corporation, Primeton Technologies Ltd., Rogue Wave Software, SAP AG.,
 * Software AG., Sun Microsystems, Sybase Inc., Xcalia, Zend Technologies,
 * 2005-2008. All rights reserved.
 *
 * </copyright>
 *
 */
package commonj.sdo.helper;

import commonj.sdo.impl.Environment;

/**
 * Bootstrap class that provides access to a HelperContext and HelperContextFactory.
 * @since SDO 3.0
 */
public class SDO
{
    /**
     * <p>Return the default HelperContextFactory.<p>
     * <p>The default HelperContextFactory is dependent upon the implementation of Environment used.</p>
     * @return a HelperContextFactory object
     */
    public static HelperContextFactory getHelperContextFactory() {
        HelperContextFactory helperContextFactory = Environment.getEnvironment().getHelperContextFactory();
        if (null == helperContextFactory) {
            throw new RuntimeException("No SDO Provider Found");
        }
        return helperContextFactory;
    }

    /**
     * <p>Return the HelperContextFactory with the specified implementation class.</p>
     * @param implName - The Java class name of the HelperContextFactory
     * @return a HelperContextFactory object
     */
    public static HelperContextFactory getHelperContextFactory(String implName) {
        if (null == implName || "".equals(implName)) {
            return Environment.getEnvironment().getHelperContextFactory();
        } else {
            return Environment.getEnvironment().getHelperContextFactory(implName);
        }
    }

    /**
     * Gets the default HelperContext.
     * @return a HelperContext object
     */
    public static HelperContext getDefaultHelperContext() {
        return Environment.getEnvironment().getDefaultHelperContext();
    }

    /**
     * Gets the HelperContext based on the identifier.
     * @param identifier - The unique identified that was used when the HelperContext was created.
     * @return a HelperContext object
     */
    public static HelperContext getHelperContext(String identifier) {
        if (null == identifier || "".equals(identifier)) {
            return Environment.getEnvironment().getDefaultHelperContext();
        } else {
            return Environment.getEnvironment().getHelperContext(identifier);
        }
    }
}
