/**
 * <copyright>
 *
 * Service Data Objects
 * Version 3.0
 * Licensed Materials
 *
 * (c) Copyright BEA Systems, Inc., International Business Machines Corporation,
 * Oracle Corporation, Primeton Technologies Ltd., Rogue Wave Software, SAP AG.,
 * Software AG., Sun Microsystems, Sybase Inc., Xcalia, Zend Technologies,
 * 2005-2008. All rights reserved.
 *
 * </copyright>
 *
 */

package commonj.sdo.impl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Constructor;
import java.security.AccessController;
import java.security.PrivilegedAction;

import commonj.sdo.helper.HelperContextFactory;

public class ServiceResolver {

    protected static final String HELPER_CONTEXT_FACTORY = "commonj.sdo.helper.HelperContextFactory";

    protected static final String ENVIRONMENT = "commonj.sdo.impl.Environment";

    protected static final String SERVICE_RESOURCE = "META-INF/services/";

    public static Environment getEnvironment() {
        ClassLoader classLoader = getContextClassLoader();
        if (classLoader == null) {
            classLoader = Environment.class.getClassLoader();
        }
        String implName = getImplementationName(ENVIRONMENT);
        if(null == implName) {
            try {
                implName = getImplementationName(classLoader, ENVIRONMENT);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            if(null == implName) {
                return new Environment.DefaultImpl();
            }
        }
        try {
            return (Environment) classLoader.loadClass(implName).newInstance();
        } catch(Exception e) {
            throw new RuntimeException(e);
        }
    }

    public static HelperContextFactory getHelperContextFactory(Environment env) {
        return getHelperContextFactory(env, null);
    }
    
    public static HelperContextFactory getHelperContextFactory(Environment env, String implName) {
        ClassLoader classLoader = getContextClassLoader();
        if (implName == null) {
            implName = getImplementationName(HELPER_CONTEXT_FACTORY);
            if(null == implName) {
                try {
                    implName = getImplementationName(classLoader, HELPER_CONTEXT_FACTORY);
                    if (null == implName) {
                        classLoader = env.getClass().getClassLoader();
                        implName = getImplementationName(classLoader, HELPER_CONTEXT_FACTORY);
                    }
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
                if(null == implName) {
                    throw new RuntimeException("HelperContextFactory implementation not found.");
                }
            }
        }
        try {
            Class hfClass = classLoader.loadClass(implName);
            Constructor hfConstructor = hfClass.getConstructor(new Class[]{Environment.class});
            return (HelperContextFactory) hfConstructor.newInstance(new Object[]{env});
        } catch(Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    /**
     * Get the implementation name from the System property.
     */
    protected static String getImplementationName(final String systemProperty) {
        try {
          return (String)AccessController.doPrivileged(new PrivilegedAction() {
              public Object run() {
                  return System.getProperty(systemProperty);
              }
          });
        } catch (SecurityException e) {
            return null;
        }
    }

    /**
     * Get the implementation name from "META-INF/services/".
     */
    protected static String getImplementationName(ClassLoader cl, String serviceProvider) throws IOException {
        InputStream is = cl.getResourceAsStream(SERVICE_RESOURCE + serviceProvider);
        if (is == null) return null;
        InputStreamReader in = new InputStreamReader(is, "UTF-8");
        BufferedReader reader = new BufferedReader(in, 128);
        try {
            String line;
            while ((line = reader.readLine()) != null) {
                int i = line.indexOf('#');
                if (i != -1) {
                    line = line.substring(0, i);
                }
                line = line.trim();
                if (line.length() > 0) return line;
            }
            return null;
        } finally {
            reader.close();
        }
    }

    protected static ClassLoader getContextClassLoader() {
        try {
          return (ClassLoader)AccessController.doPrivileged(new PrivilegedAction() {
              public Object run() {
                  return Thread.currentThread().getContextClassLoader();
              }
          });
        } catch (SecurityException e) {
            return ServiceResolver.class.getClassLoader();
        }
    }

}
